import dask
from dask.distributed import Client


def execute(ENDPOINT, values, model, model_name):
    import arches_graphql_client
    import asyncio

    client = arches_graphql_client.ResourceClient(ENDPOINT, model, model_name)
    client.connect(timeout=300)
    results = asyncio.run((client.bulk_create)(values))
    return results


def _install():
    import os

    os.system(
        "pip install -q lz4 arches_graphql_client graphene gql aiohttp dateparser python-slugify gql openpyxl pandas tqdm aiohttp"
    )


def install():
    dask.delayed(_install)().compute()


def bulk_create_from_groups(
    groups,
    cluster,
    ENDPOINT,
    detach=False,
    model="monument",
    model_name="monument_name",
):
    results = []

    with Client(cluster) as dask_client:
        dask_client.submit(_install).result()
        for n, group in enumerate(groups):
            print(f"{n} / {len(groups)}")
            fut = dask_client.submit(execute, ENDPOINT, group, model, model_name)
            if detach:
                dask.distributed.fire_and_forget(fut)
            else:
                results += fut.result()

    return results



from gql import gql, Client
import uuid
from gql.transport.aiohttp import AIOHTTPTransport
import json
import pandas as pd
from abc import ABCMeta, abstractmethod


class BaseClient(metaclass=ABCMeta):
    client = None
    root = None

    def __init__(self, root):
        self.root = root

    @property
    @abstractmethod
    def __url_prefix__(self):
        ...

    def connect(self, timeout=30):
        transport = AIOHTTPTransport(url=f"{self.root}{self.__url_prefix__}")
        self.client = Client(
            transport=transport,
            fetch_schema_from_transport=True,
            execute_timeout=timeout,
        )



import pandas as pd

from .utils import camel


class Mapper:
    def __init__(self, mapping, computed_fields, mapping_types, type_to_type, skip):
        self.mapping = mapping
        self.computed_fields = computed_fields
        self.mapping_types = mapping_types
        self.type_to_type = type_to_type
        self.skip = skip

    def map_row(self, row):
        mapping = self.mapping
        row = dict(row.items())
        row.update(
            dict(
                {
                    mapping.get(key, key): val
                    for key, val in row.items()
                    if pd.notna(val) and mapping.get(key, key) not in self.skip
                },
                **{key: None for key in self.computed_fields},
            )
        )
        return row

    def _map_field(self, field, value, row):
        if field in self.computed_fields:
            return self.computed_fields[field][0](row)
        return self.mapping_types[field][1](value)

    def get_type(self, field):
        if field in self.computed_fields:
            return self.type_to_type[self.computed_fields[field][1]]
        return self.type_to_type[self.mapping_types[field][0]]

    def get_maplist(self):
        return list(self.mapping.items()) + list(
            zip(self.computed_fields.keys(), self.computed_fields.keys())
        )

    def apply(self, row):
        kwmapping = [v for k, v in self.get_maplist() if v not in self.skip]

        field_set = self.map_row(row)
        values = {
            camel(field): self._map_field(field, value, field_set)
            for field, value in field_set.items()
            if field in kwmapping
        }
        return values

    async def bulk_create_prepare(self, rows):
        kwmapping = [v for k, v in self.get_maplist() if v not in self.skip]

        mapped = [self.map_row(row) for ix, row in rows.iterrows()]
        values = []
        for n, field_set in enumerate(mapped):
            try:
                values.append(
                    {
                        camel(field): self._map_field(field, value, field_set)
                        for field, value in field_set.items()
                        if field in kwmapping
                    }
                )
            except Exception as exc:
                raise RuntimeError(f"Could not map fieldset {n}: {field_set}") from exc

        return values

    async def bulk_create_monuments_from_rows(self, client, rows):
        values = self.bulk_create_prepare(rows)
        return (await client.bulk_create(values))["bulkCreateMonument"]["monuments"]

    async def bulk_create_monuments_from_split_df(self, client, split_df):
        results = []
        for batch, rows in enumerate(split_df):
            print(f"Batch {batch}")
            try:
                bulk_create = await self.bulk_create_monuments_from_rows(client, rows)
                results += bulk_create
            except Exception:
                print(
                    f"Exception in batch {batch} from row {batch * len(rows)} to row {(batch + 1) * len(rows)}"
                )
                raise
        return results



import json

from gql import gql

from .client import BaseClient
from .utils import camel, studly


class ResourceModelClient(BaseClient):
    __url_prefix__ = "resource-models/"
    client = None

    async def get_available_resource_models(self):

        get_query = gql(
            """
            query {
                getAvailableGraphs
            }
        """
        )
        return await self.client.execute_async(get_query)

    async def get_graph(self, resource_model):
        get_graph_query = gql(
            """
            query ($resourceModel: String) {
                buildGraph(resourceModel: $resourceModel)
            }
        """
        )
        retval = await self.client.execute_async(
            get_graph_query, variable_values={"resourceModel": resource_model}
        )
        graph = json.loads(retval["buildGraph"])
        return graph



import re
from functools import partial
import slugify

def snake(class_name):
    class_name = class_name[0].lower() + class_name[1:]
    if class_name[0] == "_" and class_name[1].isnumeric():
        class_name = class_name[1:]
    class_name = class_name[0] + re.sub("([a-zA-Z])([A-Z])", r"\1_\2", class_name[1:])
    return class_name.lower()


def string_to_enum(string):
    string = string.replace("& ", "and ")
    if "/" in string:
        return "".join(map(string_to_enum, string.split("/")))
    return studly(slugify.slugify(string.replace(" ", "-")).replace("-", " "))


def camel(string, studly=False):
    string = ((string[0].upper() if studly else string[0].lower()) + string.replace("_", " ").title()[1:]).replace(" ", "")
    if string[-1] != string[-1].lower():
        string += "_"
    if string[0].isnumeric():
        string = "_" + string
    return string
studly = partial(camel, studly=True)



import re
from functools import partial
import csv
import shapely.geometry, shapely.ops
import pandas as pd
from pyproj import CRS, Transformer
import dateparser
import geojson
from functools import partial
import pandas as pd
import dateparser

from .mapper import Mapper
from .utils import studly


def naive_date_to_date_string(date):
    return dateparser.parse(date).strftime("%Y-%m-%d")


def naive_date_to_isoformat(date):
    return dateparser.parse(date).isoformat()


def shape_to_geojson(shape, remap=None):
    if not shape or pd.isna(shape):
        return None
    # would be best to cache but for now it's throwing pickling error
    if remap:
        igr = CRS(remap[0])
        wgs84 = CRS(remap[1])
        project = Transformer.from_crs(igr, wgs84, always_xy=True).transform
        destination = shapely.ops.transform(project, shape)
    else:
        destination = shape
    transformed = geojson.FeatureCollection([geojson.Feature(geometry=destination)])
    return geojson.dumps(transformed)


mapping = {}
mapping_types_standardization = {
    "text": "str",
    "dropdown": "concept",
    "location": "location",
    "option select": "concept",
    "[text]": "[str]",
}
fields_to_coalesce = {}


def _coalesce_to_array(row, target, cb):
    return [
        val
        for val in [
            cb(row[field])
            for field in fields_to_coalesce[target]
            if pd.notna(row[field])
        ]
        if val is not None
    ]


class Loader:
    types = None

    def __init__(self, location_remap=None):
        self.location_remap = location_remap
        self._set_types()

    def _set_types(self):
        self.types: dict = dict(
            {
                "int": int,
                "str": str,
                "tuple": tuple,
                "list": list,
                "dict": dict,
                "bool": bool,
                "date": naive_date_to_date_string,
                "datetime": naive_date_to_isoformat,
                "location": lambda *args, **kwargs: shape_to_geojson(
                    *args, remap=self.location_remap, **kwargs
                ),
            }
        )

    def mapping_to_mapper(
        self,
        arches_field_name,
        source_field_name,
        fields,
        computed_fields,
        concept_lookup,
        skip,
    ):
        mapping_types = {}
        for row in fields:
            if (
                row[arches_field_name] in computed_fields
                and row[arches_field_name] not in fields_to_coalesce
            ):
                continue
            if "/" in row[arches_field_name]:
                field, _ = row[arches_field_name].split("/", -1)
                assert (
                    "/" not in field
                ), "Can only handle one level of grouping (semantic field)"
                assert (
                    field in computed_fields
                ), f"Cannot automatically ingest semantic fields, you must create a computed field for {field}"
                continue

            typ = re.sub("\[(.*)\]", r"\1", row["type"])
            is_array = typ != row["type"]
            typ = mapping_types_standardization.get(typ, typ) or "str"

            if typ == "concept":
                typ = row[arches_field_name]
                mapping_cb = concept_lookup[row[arches_field_name]]
            elif typ in self.types:
                mapping_cb = self.types[typ]
            else:
                print(f"Skipping unknown type: {typ} for {row[source_field_name]}")

            if is_array:
                typ = f"[{typ}]"
                fields_to_coalesce.setdefault(row[arches_field_name], [])
                fields_to_coalesce[row[arches_field_name]].append(
                    row[source_field_name]
                )
                if row[arches_field_name] not in computed_fields:
                    computed_fields[row[arches_field_name]] = (
                        partial(
                            _coalesce_to_array,
                            target=row[arches_field_name],
                            cb=mapping_cb,
                        ),
                        typ,
                    )
                else:
                    assert (
                        computed_fields[row[arches_field_name]][1] == typ
                    ), f"{row['map_name_arches']} {typ}"
            else:
                mapping_types[row[arches_field_name]] = (typ, mapping_cb)

        mapping = {}
        for row in fields:
            if row[arches_field_name] != "" and "/" not in row[arches_field_name]:
                mapping[row[source_field_name]] = row[arches_field_name]

        type_to_type = {
            "str": "String",
            "date": "String",
            "datetime": "String",
            "location": "JSONString",
        }
        for concept in concept_lookup:
            type_to_type[concept] = studly(concept)
            type_to_type[f"[{concept}]"] = f"[{studly(concept)}]"

        mapper = Mapper(mapping, computed_fields, mapping_types, type_to_type, skip)
        return mapper



import json
from pathlib import Path
import csv
import sys

mapping_types_standardization = {
    "str": "str",
    "integer": "int",
    "int": "int",
    "text": "str",
    "concept": "concept",
    "date": "date",
    "datetime": "datetime",
    "text": "str",
    "location": "geojson",
    "dropdown": "concept",
    "[text]": "[str]",
    "[concept]": "[concept]",
    "option select": "concept",
}

def update_mapping_new(arches_model, filename):
    model_definition = arches_model['graph'][0]

    nodes = {m["alias"]: m for m in model_definition["nodes"]}
    parents = {m["nodegroupid"]: m["parentnodegroup_id"] for m in model_definition["nodegroups"] if m["parentnodegroup_id"]}
    models = {}
    with open(filename, "r") as fd:
        reader = csv.DictReader(fd)
        for row in reader:
            node = nodes[row["map_name_arches"].split("/", -1)[-1]]
            if row["type"] not in mapping_types_standardization:
                print("Missing", row["type"])
                continue
            if row["map_name_arches"]:
                typ = row["type"]
                models[row["map_name_arches"]] = {
                    "lang": row.get("Language", "en"),
                    "type": mapping_types_standardization[typ],
                    "nodeid": node["nodeid"],
                    "nodegroupid": node["nodegroup_id"],
                }
                assert node["nodeid"] == row["nodeid"]
                assert node["nodegroup_id"] == row["nodegroup_id"]
                if row["nodegroup_id"] in parents:
                    models[row["map_name_arches"]]["parentnodegroup_id"] = parents[node["nodegroup_id"]]

    models["graphid"] = model_definition["graphid"]
    return models

def add_model(model, mapping_file, api_file, model_folder="./pkg/graphs/resource_models/"):
    with (Path(model_folder) / f"{model}.json").open("r") as f:
        arches_model = json.load(f)
    if api_file.exists:
        with filename_out.open("r") as fd:
            models = json.load(fd)
    else:
        models = {}
    models[model] = update_mapping_new(arches_model, mapping_file)
    with api_file.open("w") as fd:
        json.dump(models, fd)

if __name__ == "__main__":
    model = sys.argv[3]
    filename_out = Path(sys.argv[2])
    filename_in = Path(sys.argv[1])
    add_model(model, filename_in, filename_out)



from gql import gql

from .client import BaseClient
from .utils import camel, studly


class ResourceClient(BaseClient):
    __url_prefix__ = "resources/"
    client = None

    def __init__(self, root, resource_model_name, label_field):
        super().__init__(root)
        self.resource_model_name = resource_model_name
        self.label_field = label_field

    async def bulk_create(self, field_sets):
        query = gql(
            f"""
            mutation bulkCreate{studly(self.resource_model_name)}($input: [{studly(self.resource_model_name)}Input]) {{
                bulkCreate{studly(self.resource_model_name)}(fieldSets: $input) {{
                    ok,
                    {camel(self.resource_model_name)}s {{ id, {camel(self.label_field)} }}
                }}
            }}
            """
        )

        results = await self.client.execute_async(
            query, variable_values={"input": field_sets}
        )
        return results[f"bulkCreate{studly(self.resource_model_name)}"][
            f"{camel(self.resource_model_name)}s"
        ]

    async def get(self, id, fields=None):
        if not fields:
            fields = [self.label_field]
        get_query = gql(
            f"""
            query ($id: UUID!) {{
                get{studly(self.resource_model_name)} (id: $id) {{
                    {', '.join(field for field in fields)}
                }}
            }}
        """
        )
        return await self.client.execute_async(get_query, variable_values={"id": id})

    async def search(self, text, fields):
        search_query = gql(
            f"""
            query ($text: String, $fields: [String]) {{
              search{studly(self.resource_model_name)}(text: $text, fields: $fields) {{
                  id
                  {camel(self.resource_model_name)}
              }}
            }}
        """
        )
        return await self.client.execute_async(
            search_query, variable_values={"text": text, "fields": fields}
        )

    # Execute the query on the transport
    async def delete(self, id):
        delete_query = gql(
            f"""
            mutation ($id: UUID) {{
              delete{studly(self.resource_model_name)}(id: $id) {{
                ok
              }}
            }}
        """
        )
        return await self.client.execute_async(delete_query, variable_values={"id": id})



import uuid
from gql import gql

from .client import BaseClient
from .utils import camel, studly


class ConceptClient(BaseClient):
    __url_prefix__ = "concepts/"

    async def get_concept_details(self, name_or_id):
        get_terms_query = gql(
            "query ($concept: String) { getConceptDetails(nameOrId: $concept) }"
        )
        return (
            await self.client.execute_async(
                get_terms_query, variable_values={"concept": name_or_id}
            )
        )["getConceptDetails"]

    async def get_concept(self, name_or_id):
        get_terms_query = gql(
            "query ($concept: String) { getConcept(nameOrId: $concept) { id, label, nodetype } }"
        )
        return (
            await self.client.execute_async(
                get_terms_query, variable_values={"concept": name_or_id}
            )
        )["getConcept"]

    async def get_available_concepts(self):
        get_available_concepts_query = gql(
            """
            query {
                getAvailableConcepts
            }
        """
        )
        return sorted(
            (await self.client.execute_async(get_available_concepts_query))[
                "getAvailableConcepts"
            ]
        )

    async def get_term_list(self, concept):
        get_terms_query = gql(
            "query ($concept: String) { getTermList(conceptNameOrId: $concept) }"
        )
        return (
            await self.client.execute_async(
                get_terms_query, variable_values={"concept": concept}
            )
        )["getTerms"]

    async def get_terms(self, concept):
        get_terms_query = gql(
            "query ($concept: String) { getTerms(conceptNameOrId: $concept) { label, fullLabel, identifier } }"
        )
        return [
            term
            for term in (
                await self.client.execute_async(
                    get_terms_query, variable_values={"concept": concept}
                )
            )["getTerms"]
        ]

    async def add_term(self, concept, label_or_full_label):
        if isinstance(label_or_full_label, list):
            identifier = "》".join(label_or_full_label)
        else:
            identifier = label_or_full_label

        if isinstance(concept, str):
            try:
                concept = {"id": uuid.UUID(concept)}
            except ValueError:
                concept = {"label": concept}
        add_term_mut = gql(
            "mutation ($identifier: String, $concept: ConceptInput) { addTerm(identifier: $identifier, concept: $concept) { ok } }"
        )
        return (
            await self.client.execute_async(
                add_term_mut,
                variable_values={"identifier": identifier, "concept": concept},
            )
        )["addTerm"]
